// SVN.h : Declaration of the SVN

#pragma once
#include "resource.h"       // main symbols
#include "atlsimpstr.h"

#include "apr_general.h"
#include "svn_pools.h"
#include "svn_client.h"
#include "svn_sorts.h"
#include "svn_path.h"
#include "svn_wc.h"


#include "SVNStatus.h"
#include "SVNProperties.h"



#define DEFAULT_ARRAY_SIZE 5


svn_error_t * svn_cl__get_log_message (const char **log_msg,
									apr_array_header_t * commit_items,
									void *baton, apr_pool_t * pool);

/**
 * enum for the kind of notification event.
 */
[export]
typedef enum 
{
	notify_add = 0,
	notify_copy,
	notify_delete,
	notify_restore,
	notify_revert,
	notify_resolve,

	/* The update actions are also used for checkouts, switches, and merges. */
	notify_update_delete,     /* Got a delete in an update. */
	notify_update_add,        /* Got an add in an update. */
	notify_update_update,     /* Got any other action in an update. */
	notify_update_completed,  /* The last notification in an update */
	notify_update_external,   /* About to update an external module;
										use for checkouts and switches too,
										end with svn_wc_update_completed. */

	notify_commit_modified,
	notify_commit_added,
	notify_commit_deleted,
	notify_commit_replaced,
	notify_commit_postfix_txdelta
} notify_action;

/**
 * enum for the state of the object when the notification event is fired.
 */
[export]
typedef enum 
{
	notify_state_inapplicable = 0,
	notify_state_unknown,     /* Notifier doesn't know or isn't saying. */
	notify_state_unchanged,   /* The state did not change. */
	notify_state_modified,    /* Pristine state was modified. */
	notify_state_merged,      /* Modified state had mods merged in. */
	notify_state_conflicted   /* Modified state got conflicting mods. */
} notify_state;

/**
 * enum for subversion node types.
 */
[export]
typedef enum
{
	node_none,        /* absent */
	node_file,        /* regular file */
	node_dir,         /* directory */
	node_unknown      /* something's here, but we don't know what */
} node_kind;


// ISVN
[
	object,
	uuid("86FC0947-8F37-4EF8-847B-D6C09DD473A7"),
	dual,	helpstring("ISVN Interface"),
	pointer_default(unique)
]
__interface ISVN : IDispatch
{
	[id(1), helpstring("returns the svn status object of a file")] HRESULT GetFileStatus([in] VARIANT_BOOL update, [in] BSTR path, [out,retval] IDispatch** StatusObj);
	[id(2), helpstring("returns the SVNProperties object")] HRESULT GetProperties([in] BSTR path, [out,retval] IDispatch** PropertiesObj);
	[id(3), helpstring("checks out a working copy of a repository"), source] HRESULT Checkout([in] BSTR moduleName, [in] BSTR destPath, [in] LONG revision, [in] VARIANT_BOOL recurse);
	[id(4), helpstring("sets a file for deletion"), source] HRESULT Remove([in] BSTR path, [in] VARIANT_BOOL force);
	[id(5), helpstring("reverts a file to a pristine state"), source] HRESULT Revert([in] BSTR path, [in] VARIANT_BOOL recurse);
	[id(6), helpstring("adds a file to the repository"), source] HRESULT Add([in] BSTR path, [in] VARIANT_BOOL recurse);
	[id(7), helpstring("updates the directory with the repository"), source] HRESULT Update([in] BSTR path, [in] LONG revision, [in] VARIANT_BOOL recurse);
	[id(8), helpstring("commits all changes to the repository"), source] HRESULT Commit([in] BSTR path, [in] BSTR message, [in] VARIANT_BOOL recurse, [out,retval] LONG* revision);
	[id(9), helpstring("copies a versioned file with the history preserved"), source] HRESULT Copy([in] BSTR srcPath, [in] BSTR destPath, [in] LONG revision);
	[id(10), helpstring("moves/renames a file"), source] HRESULT Move([in] BSTR srcPath, [in] BSTR destPath, [in] VARIANT_BOOL force);
	[id(11), helpstring("creates a new directory"), source] HRESULT MakeDir([in] BSTR path, [in] BSTR message);
	[id(12), helpstring("recursively cleans up a local directory, finishing incomplete operations, removing lockfiles etc.")] HRESULT CleanUp([in] BSTR path);
	[id(13), helpstring("removes the conflicted state of a file")] HRESULT Resolve([in] BSTR path, [in] VARIANT_BOOL recurse);
	[id(14), helpstring("exports a repository into a clean directory without administrative files"), source] HRESULT Export([in] BSTR srcPath, [in] BSTR destPath, [in] LONG revision);
	[id(15), helpstring("switches local copy to a different repository/branch"), source] HRESULT Switch([in] BSTR path, [in] BSTR url, [in] LONG revision, [in] VARIANT_BOOL recurse);
	[id(16), helpstring("imports a local directory into a repository"), source] HRESULT Import([in] BSTR path, [in] BSTR url, [in] BSTR newEntry, [in] BSTR message, [in] VARIANT_BOOL recurse);
	[id(17), helpstring("merges two repositories/branches into one"), source] HRESULT Merge([in] BSTR path1, [in] LONG revision1, [in] BSTR path2, [in] LONG revision2, [in] BSTR localPath, [in] VARIANT_BOOL force, [in] VARIANT_BOOL recurse);
	[id(18), helpstring("receives log messages and fires Log-event for every message"), source] HRESULT ReceiveLog([in] BSTR path, [in] LONG revisionStart, [in] LONG revisionEnd, [in] VARIANT_BOOL reverse);
};


// _ISVNEvents
[
	dispinterface,
	uuid("4BB80F5E-37AC-4A62-BF09-1CE5DCFC8639"),
	helpstring("_ISVNEvents Interface")
]
__interface _ISVNEvents
{
	[id(1), helpstring("notify event")] HRESULT Notify([in] BSTR path, [in] notify_action action, [in] node_kind kind, [in] BSTR mime_type, [in] notify_state content, [in] notify_state prop, [in] LONG revision);
	[id(2), helpstring("prompt callback used to ask for authenticating information")] HRESULT Prompt([out] BSTR* info, [in] BSTR prompt, [in] VARIANT_BOOL hide);
	[id(3), helpstring("called for every log message")] HRESULT Log([in] LONG revnum, [in] BSTR author, [in] BSTR date, [in] BSTR message);
};


/**
 * \ingroup SVNCOM
 *
 * SVN object. This object provides (a part of) the client library
 * of Subversion (see http://subversion.tigris.org) as a COM object.
 *
 * \version 1.0
 * first version
 *
 * \date 09-27-2002
 *
 * \author kueng
 *
 * \par license
 * This code is absolutely free to use and modify. The code is provided "as is" with
 * no expressed or implied warranty. The author accepts no liability if it causes
 * any damage to your computer, causes your pet to fall ill, increases baldness
 * or makes your car start emitting strange noises when you start it up.
 * This code has no bugs, just undocumented features!
 * 
 * \todo 
 *
 * \bug 
 *
 */
[
	coclass,
	threading("apartment"),
	support_error_info("ISVN"),
	event_source("com"),
	vi_progid("SVNCOM.SVN"),
	progid("SVNCOM.SVN.1"),
	version(1.0),
	uuid("C3E93C03-E982-48E0-B883-BD6B3A889F56"),
	helpstring("Subversion object")
]
class ATL_NO_VTABLE SVN : 
	public ISVN
{
public:
	SVN()
	{
	}

	__event __interface _ISVNEvents;

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		pool = svn_pool_create (NULL);
		auth_obj = (svn_client_auth_baton_t *) apr_pcalloc 
					(pool, sizeof (*auth_obj));

		auth_obj->prompt_callback = prompt;
		auth_obj->prompt_baton = this;
		auth_obj->store_auth_info = true;
		auth_obj->got_new_auth_info = true;

		return S_OK;
	}
	
	void FinalRelease() 
	{
		svn_pool_destroy (pool);
	}

public:

	/**
	 * Builds the status object of a file or directory.
	 * \param update set to true if the status should be updated with the repository. Needs network access.
	 * \param path the path to the file or directory to build the status on
	 * \return StatusObj the status object
	 */
	STDMETHOD(GetFileStatus)(VARIANT_BOOL update, BSTR path, IDispatch** StatusObj);
	/**
	 * Creates a Properties object which can be used to read and write the properties of a
	 * file or directory.
	 * \param path the path to the file or directory for the properties object
	 * \return PropertiesObj the properties object
	 */
	STDMETHOD(GetProperties)(BSTR path, IDispatch** PropertiesObj);
	/** 
	 * Checkout a working copy of moduleName at revision, using destPath as the root
	 * directory of the newly checked out working copy
	 * \param moduleName the path/url of the repository
	 * \param destPath the path to the local working copy
	 * \param revision the revision number to check out
	 * \param recurse TRUE if you want to check out all subdirs and files (recommended)
	 */
	STDMETHOD(Checkout)(BSTR moduleName, BSTR destPath, LONG revision, VARIANT_BOOL recurse);
	/**
	 * If path is a URL, use the MESSAGE to immediately attempt 
	 * to commit a deletion of the URL from the repository. 
	 * Else, schedule a working copy path for removal from the repository.
	 * path's parent must be under revision control. This is just a
	 * \b scheduling operation.  No changes will happen to the repository until
	 * a commit occurs.  This scheduling can be removed with
	 * Revert(). If path is a file it is immediately removed from the
	 * working copy. If path is a directory it will remain in the working copy
	 * but all the files, and all unversioned items, it contains will be
	 * removed. If force is not set then this operation will fail if path
	 * contains locally modified and/or unversioned items. If force is set such
	 * items will be deleted.
	 *
	 * \param path the file/directory to delete
	 * \param force if TRUE, all files including those not versioned are deleted. If FALSE the operation
	 * will fail if a directory contains unversioned files or if the file itself is not versioned.
	 */
	STDMETHOD(Remove)(BSTR path, VARIANT_BOOL force);
	/**
	 * Reverts a file/directory to its pristine state. I.e. its reverted to the state where it
	 * was last updated with the repository.
	 * \param path the file/directory to revert
	 * \param recurse 
	 */
	STDMETHOD(Revert)(BSTR path, VARIANT_BOOL recurse);
	/**
	 * Schedule a working copy path for addition to the repository.
	 * path's parent must be under revision control already, but path is
	 * not. If recursive is set, then assuming path is a directory, all
	 * of its contents will be scheduled for addition as well.
	 * Important:  this is a \b scheduling operation.  No changes will
	 * happen to the repository until a commit occurs.  This scheduling
	 * can be removed with Revert().
	 *
	 * \param path the file/directory to add 
	 * \param recurse 
	 */
	STDMETHOD(Add)(BSTR path, VARIANT_BOOL recurse);
	/**
	 * Update working tree path to revision.
	 * \param path the file/directory to update
	 * \param revision the revision the local copy should be updated to or -1 for HEAD
	 * \param recurse 
	 */
	STDMETHOD(Update)(BSTR path, LONG revision, VARIANT_BOOL recurse);
	/**
	 * Commit file or directory path into repository, using message as
	 * the log message.
	 * 
	 * If no error is returned and revision is set to -1, then the 
	 * commit was a no-op; nothing needed to be committed.
	 *
	 * \remark to specify multiple sources separate the paths with a ";"
	 *
	 * \param path the file/directory to commit
	 * \param message a log message describing the changes you made
	 * \param recurse 
	 * \param revision the resulting revision number. return value.
	 */
	STDMETHOD(Commit)(BSTR path, BSTR message, VARIANT_BOOL recurse, LONG* revision);
	/**
	 * Copy srcPath to destPath.
	 * 
	 * srcPath must be a file or directory under version control, or the
	 * URL of a versioned item in the repository.  If srcPath is a URL,
	 * revision is used to choose the revision from which to copy the
	 * srcPath. destPath must be a file or directory under version
	 * control, or a repository URL, existent or not.
	 *
	 * If either srcPath or destPath are URLs, immediately attempt 
	 * to commit the copy action in the repository. 
	 * 
	 * If neither srcPath nor destPath is a URL, then this is just a
	 * variant of Add(), where the destPath items are scheduled
	 * for addition as copies.  No changes will happen to the repository
	 * until a commit occurs.  This scheduling can be removed with
	 * Revert().
	 *
	 * \param srcPath source path
	 * \param destPath destination path
	 */
	STDMETHOD(Copy)(BSTR srcPath, BSTR destPath, LONG revision);
	/**
	 * Move srcPath to destPath.
	 * 
	 * srcPath must be a file or directory under version control and 
	 * destPath must also be a working copy path (existent or not).
	 * This is a scheduling operation.  No changes will happen to the
	 * repository until a commit occurs.  This scheduling can be removed
	 * with Revert().  If srcPath is a file it is removed from
	 * the working copy immediately.  If srcPath is a directory it will
	 * remain n the working copy but all the files, and unversioned items,
	 * it contains will be removed.
	 *
	 * If srcPath contains locally modified and/or unversioned items and
	 * force is not set, the copy will fail. If force is set such items
	 * will be removed.
	 * 
	 * \param srcPath source path
	 * \param destPath destination path
	 * \param revision 
	 * \param force 
	 */
	STDMETHOD(Move)(BSTR srcPath, BSTR destPath, VARIANT_BOOL force);
	/**
	 * If path is a URL, use the message to immediately
	 * attempt to commit the creation of the directory URL in the
	 * repository.
	 * Else, create the directory on disk, and attempt to schedule it for
	 * addition.
	 * 
	 * \param path 
	 * \param message 
	 */
	STDMETHOD(MakeDir)(BSTR path, BSTR message);
	/**
	 * Recursively cleanup a working copy directory DIR, finishing any
	 * incomplete operations, removing lockfiles, etc.
	 * \param path the file/directory to clean up
	 */
	STDMETHOD(CleanUp)(BSTR path);
	/**
	 * Remove the 'conflicted' state on a working copy PATH.  This will
	 * not semantically resolve conflicts;  it just allows path to be
	 * committed in the future.
	 * If recurse is set, recurse below path, looking for conflicts to
	 * resolve.
	 * If path is not in a state of conflict to begin with, do nothing.
	 * 
	 * \param path the path to resolve
	 * \param recurse 
	 */
	STDMETHOD(Resolve)(BSTR path, VARIANT_BOOL recurse);
	/**
	 * Export the contents of either a subversion repository or a subversion 
	 * working copy into a 'clean' directory (meaning a directory with no 
	 * administrative directories).

	 * \param srcPath	either the path the working copy on disk, or a url to the
	 *					repository you wish to export.
	 * \param destPath	the path to the directory where you wish to create the exported tree.
	 * \param revision	the revision that should be exported, which is only used 
	 *					when exporting from a repository.
	 */
	STDMETHOD(Export)(BSTR srcPath, BSTR destPath, LONG revision);
	/**
	 * Switch working tree path to URL at revision
	 *
	 * Summary of purpose: this is normally used to switch a working
	 * directory over to another line of development, such as a branch or
	 * a tag.  Switching an existing working directory is more efficient
	 * than checking out URL from scratch.
	 *
	 * \param path the path of the working directory
	 * \param url the url of the repository
	 * \param revision the revision number to switch to
	 * \param recurse 
	 */
	STDMETHOD(Switch)(BSTR path, BSTR url, LONG revision, VARIANT_BOOL recurse);
	/**
	 * Import file or directory path into repository directory url at
	 * head and using LOG_MSG as the log message for the (implied)
	 * commit.
	 * If path is a directory, the contents of that directory are
	 * imported, under a new directory named newEntry under url; or if
	 * newEntry is null, then the contents of path are imported directly
	 * into the directory identified by url.  Note that the directory path
	 * itself is not imported -- that is, the basename of path is not part
	 * of the import.
	 *
	 * If path is a file, that file is imported as newEntry (which may
	 * not be null).
	 *
	 * In all cases, if newEntry already exists in url, return error.
	 *
	 * \param path		the file/directory to import
	 * \param url		the url to import to
	 * \param newEntry	the new entry created in the repository directory
	 *					identified by URL.
	 * \param message	log message used for the 'commit'
	 * \param recurse 
	 */
	STDMETHOD(Import)(BSTR path, BSTR url, BSTR newEntry, BSTR message, VARIANT_BOOL recurse);
	/**
	 * Merge changes from path1/revision1 to path2/revision2 into the
	 * working-copy path localPath.  path1 and path2 can be either
	 * working-copy paths or URLs.
	 *
	 * By "merging", we mean:  apply file differences and schedule 
	 * additions & deletions when appopriate.
	 *
	 * path1 and path2 must both represent the same node kind -- that is,
	 * if path1 is a directory, path2 must also be, and if path1 is a
	 * file, path2 must also be.
	 * 
	 * If recurse is true (and the paths are directories), apply changes
	 * recursively; otherwise, only apply changes in the current
	 * directory.
	 *
	 * If force is not set and the merge involves deleting locally modified or
	 * unversioned items the operation will fail.  If force is set such items
	 * will be deleted.
	 *
	 * \param path1		first path
	 * \param revision1	first revision
	 * \param path2		second path
	 * \param revision2 second revision
	 * \param localPath destination path
	 * \param force		see description
	 * \param recurse 
	 */
	STDMETHOD(Merge)(BSTR path1, LONG revision1, BSTR path2, LONG revision2, BSTR localPath, VARIANT_BOOL force, VARIANT_BOOL recurse);
	/**
	 * fires the Log-event on each log message from revisionStart
	 * to revisionEnd inclusive (but never fires the event
	 * on a given log message more than once).
	 * To receive the messages you need to listen to Log() events.
	 *
	 * \param path the file/directory to get the log of
	 * \param revisionStart	the revision to start the logs from
	 * \param revisionEnd the revision to stop the logs
	 * \param reverse TRUE if the most recent log message 
	 *					should be received first, false to
	 *					get the oldest log message first
	 */
	STDMETHOD(ReceiveLog)(BSTR path, LONG revisionStart, LONG revisionEnd, VARIANT_BOOL reverse);

private:
	svn_client_auth_baton_t *	auth_obj;
	apr_hash_t *				statushash;
	apr_array_header_t *		statusarray;
	svn_wc_status_t *			status;
	apr_pool_t *				pool;			///< memory pool
	svn_opt_revision_t			rev;			///< subversion revision. used by getRevision()
	svn_error_t *				Err;			///< Global error object struct


	svn_opt_revision_t *	getRevision (long revNumber);
	void * logMessage (const char * message, char * baseDirectory = NULL);
	void	preparePath(CString &path);
	apr_array_header_t * target (const char * path);
public:
	static svn_error_t* prompt(char **info, 
					const char *prompt, 
					svn_boolean_t hide, 
					void *baton, 
					apr_pool_t *pool);
	static void notify( void *baton,
					const char *path,
					svn_wc_notify_action_t action,
					svn_node_kind_t kind,
					const char *mime_type,
					svn_wc_notify_state_t content_state,
					svn_wc_notify_state_t prop_state,
					svn_revnum_t revision);
	static svn_error_t* logReceiver(void* baton, 
					apr_hash_t* ch_paths, 
					svn_revnum_t rev, 
					const char* author, 
					const char* date, 
					const char* msg, 
					apr_pool_t* pool);
};


